/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangBescheidWizardE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheid-wizard.e2e.component';
import { VorgangBescheideE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheide.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-subnavigation';
import {
  VorgangE2E,
  VorgangStatusE2E,
  vorgangStatusLabelE2E,
} from 'apps/alfa-e2e/src/model/vorgang';
import {
  TEST_FILE_BESCHEID_ANHANG_VALID,
  TEST_FILE_BESCHEID_VALID,
} from 'apps/alfa-e2e/src/support/data.util';
import { getAdjustedDateGerman } from 'apps/alfa-e2e/src/support/tech.util';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import {
  countDownloadFiles,
  deleteDownloadFolder,
  dropCollections,
  readFileFromDownloads,
} from '../../../support/cypress-helper';
import {
  contains,
  enterWith,
  exist,
  haveLength,
  haveText,
  notBeVisible,
  notExist,
  visible,
} from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Bescheid Info anzeigen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const abgelehntText: string = 'Abgelehnt am';
  const fortsetzenText: string = 'Bescheiden fortsetzen';
  const entwurfText: string = '(Entwurf)';

  const vorgangPage: VorgangPage = new VorgangPage();
  const bescheidWizard: VorgangBescheidWizardE2EComponent = vorgangPage.getBescheidWizard();

  const bescheidVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Wizard Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  const vorgangSubnavigationButtons: VorgangSubnavigationE2EComponent =
    vorgangPage.getSubnavigation();

  const bescheide: VorgangBescheideE2EComponent = vorgangPage.getBescheide();

  before(() => {
    initVorgaenge([bescheidVorgang]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Show Bescheid Entwurf info on details page', () => {
    it('should not show Bescheid info in list', () => {
      notExist(bescheide.getBescheidListItem());
    });

    it('should open Wizard', () => {
      vorgangList.getListItem(bescheidVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      vorgangFormularButtons.getBescheidenButton().click();

      bescheidWizard.isOpened();
    });

    it('should click abgelehnt, set date and continue to step 2', () => {
      bescheidWizard.ablehnen();
      bescheidWizard.enterDate(-1);
      bescheidWizard.weiter();
      bescheidWizard.isBescheidDocumentsStep();
    });

    it('should close Wizard', () => {
      bescheidWizard.close();
      bescheidWizard.getCloseSpeichernButton().click();
      bescheidWizard.isClosed();
    });

    it('should show status In Bearbeitung', () => {
      haveText(
        vorgangPage.getVorgangDetailHeader().getStatus(),
        vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG],
      );
    });

    it('should show Bescheid as Entwurf with abgelehnt status and date yesterday', () => {
      exist(bescheide.getBescheidContainer());
      contains(bescheide.getBescheidContainer(), abgelehntText + ' ' + getAdjustedDateGerman(-1));
      contains(bescheide.getBescheidContainer(), entwurfText);
    });

    it('should show caption Bescheid fortsetzen on button', () => {
      contains(vorgangFormularButtons.getBescheidenButton(), fortsetzenText);
    });
  });

  describe('should show complete info after saving Bescheid', () => {
    it('should set date to two days before and continue to step 2', () => {
      vorgangFormularButtons.getBescheidenButton().click();
      enterWith(bescheidWizard.getDateInput(), getAdjustedDateGerman(-2));
      bescheidWizard.getWeiterButton().click();
      bescheidWizard.isBescheidDocumentsStep();
    });

    it('should upload files and continue to step 3', () => {
      bescheidWizard.uploadBescheid(TEST_FILE_BESCHEID_VALID);
      bescheidWizard.bescheidUploadSpinnerIsClosed();
      bescheidWizard.uploadAttachment(TEST_FILE_BESCHEID_ANHANG_VALID);
      bescheidWizard.attachmentSpinnerIsClosed();
      bescheidWizard.weiter();

      bescheidWizard.isBescheidVersendenStep();
    });

    it('should save Bescheid and close wizard', () => {
      bescheidWizard.getSaveButton().click();
      bescheidWizard.getConfirmAndSaveButton().click();
      waitForSpinnerToDisappear();
      bescheidWizard.isClosed();
    });

    it('should show Bescheid as abgelehnt, and contain both files', () => {
      exist(bescheide.getBescheidContainer());

      contains(bescheide.getBescheidContainer(), abgelehntText);
      exist(bescheide.getFileBescheidValidInVorgang());
      exist(bescheide.getFileAnhangValidInVorgang());
    });

    it('should show date and Beschieden icon on top of Vorgang page', () => {
      exist(bescheide.getBeschiedenDateInHeader());
      exist(bescheide.getAbgelehntIconInHeader());
      contains(bescheide.getBeschiedenDateInHeader(), getAdjustedDateGerman(-2));
    });

    it('should (not) show Bescheid container when I open/close the panel', () => {
      bescheide.getBescheidExpansionPanel().click();
      notBeVisible(bescheide.getBescheidContainer());

      bescheide.getBescheidExpansionPanel().click();
      visible(bescheide.getBescheidContainer());
    });

    it('should download both attachments', () => {
      deleteDownloadFolder().then(() => {
        bescheide.getFileBescheidValidInVorgang().click();
        bescheide.getFileAnhangValidInVorgang().click();
        readFileFromDownloads(TEST_FILE_BESCHEID_ANHANG_VALID).then(() => {
          countDownloadFiles().then((count) => {
            expect(count).to.eq(2);
          });
        });
      });
    });
  });

  describe('should show second Bescheid after reopening Vorgang and creating new Bescheid', () => {
    it('create new Bescheid and set it to abgelehnt', () => {
      vorgangFormularButtons.getWiedereroeffnenButton().click();
      vorgangFormularButtons.getBescheidenButton().click();
      bescheidWizard.ablehnen();
      bescheidWizard.weiter();
      bescheidWizard.isBescheidDocumentsStep();
    });

    it('should upload Bescheid and continue to step 3', () => {
      bescheidWizard.uploadBescheid(TEST_FILE_BESCHEID_VALID);
      bescheidWizard.bescheidUploadSpinnerIsClosed();
      bescheidWizard.weiter();
      bescheidWizard.isBescheidVersendenStep();
    });

    it('should show 2 Bescheid containers after saving', () => {
      bescheidWizard.getSaveButton().click();
      bescheidWizard.getConfirmAndSaveButton().click();
      exist(bescheidWizard.getBescheidSaveSpinner());
      notExist(bescheidWizard.getBescheidSaveSpinner());

      haveLength(bescheide.getBescheidContainer(), 2);
    });

    it('should show info on top of Vorgang page', () => {
      exist(bescheide.getBeschiedenDateInHeader());
      exist(bescheide.getAbgelehntIconInHeader());
      contains(bescheide.getBeschiedenDateInHeader(), getAdjustedDateGerman(0));
    });
  });

  describe('show Bescheid info on Vorgang list', () => {
    it('should show Bescheid info on list page', () => {
      vorgangSubnavigationButtons.getBackButton().click();

      exist(bescheide.getBescheidListItem());
      contains(bescheide.getBescheidListItem(), 'abgelehnt');
    });
  });
});
