/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierun
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidDocument, BescheidLinkRel, createEmptyBescheidDocument } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { HasLinkPipe, StateResource } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, getElementComponentFromFixtureByCss, mock, Mock, notExistsAsHtmlElement, triggerEvent, } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheidDocument, createBescheidResource } from '../../../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { BescheidFormService } from '../../../bescheid.formservice';
import { BescheidWizardCreateDocumentButtonContainerComponent } from '../../create-document-button-container/bescheid-wizard-create-document-button-container.component';
import { BescheidWizardUploadAttachmentButtonContainerComponent } from '../../upload-attachment-button-container/bescheid-wizard-upload-attachment-button-container.component';
import { BescheidWizardUploadDocumentButtonContainerComponent } from '../../upload-document-button-container/bescheid-wizard-upload-document-button-container.component';
import { BescheidWizardWeiterButtonComponent } from '../../weiter-button/bescheid-wizard-weiter-button.component';
import { BescheidWizardDokumenteHochladenFormComponent } from './bescheid-wizard-dokumente-hochladen-form.component';

describe('BescheidWizardDokumenteHochladenFormComponent', () => {
  let component: BescheidWizardDokumenteHochladenFormComponent;
  let fixture: ComponentFixture<BescheidWizardDokumenteHochladenFormComponent>;

  const createDocumentButtonTestId: string = getDataTestIdOf('create-document-button');
  const uploadDocumentButtonTestId: string = getDataTestIdOf('upload-document-button');
  const uploadAttachmentButtonTestId: string = getDataTestIdOf('upload-attachment-button');
  const weiterButtonTestId: string = getDataTestIdOf('weiter-button');

  const bescheidDocument: BescheidDocument = createBescheidDocument();

  let formService: Mock<BescheidFormService>;

  beforeEach(() => {
    formService = mock(BescheidFormService);
    formService.submit = jest.fn().mockReturnValue(EMPTY);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardDokumenteHochladenFormComponent,
        MockComponent(BescheidWizardCreateDocumentButtonContainerComponent),
        MockComponent(BescheidWizardUploadDocumentButtonContainerComponent),
        MockComponent(BescheidWizardUploadAttachmentButtonContainerComponent),
        MockComponent(BescheidWizardWeiterButtonComponent),
        HasLinkPipe,
      ],
      providers: [{ provide: BescheidFormService, useValue: formService }],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardDokumenteHochladenFormComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  describe('component', () => {
    it('should create', () => {
      expect(component).toBeTruthy();
    });

    describe('set bescheid document', () => {
      it('should emit missing bescheid document false', () => {
        component.bescheidDocumentMissing.emit = jest.fn();
        component.bescheidDocument = bescheidDocument;

        expect(component.bescheidDocumentMissing.emit).toHaveBeenCalledWith(false);
      });

      it('should NOT emit missing bescheid document', () => {
        component.bescheidDocumentMissing.emit = jest.fn();
        component.bescheidDocument = createEmptyBescheidDocument();

        expect(component.bescheidDocumentMissing.emit).not.toHaveBeenCalled();
      });
    });

    describe('gotoNextStep', () => {
      it('should emit missing bescheid document true', () => {
        component.bescheidDocumentMissing.emit = jest.fn();
        component.bescheidDocument = createEmptyBescheidDocument();

        component.gotoNextStep();

        expect(component.bescheidDocumentMissing.emit).toHaveBeenCalledWith(true);
      });

      it('should submit form', () => {
        component.bescheidDocumentMissing.emit = jest.fn();
        component.bescheidDocument = bescheidDocument;

        component.gotoNextStep();

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should emit weiter click', () => {
        component.weiterClickEmitter.emit = jest.fn();
        component.bescheidDocument = bescheidDocument;
        formService.submit = jest.fn().mockReturnValue(of(createSuccessfullyDoneCommandStateResource()));

        component.gotoNextStep();
        component.submitStateResource$.subscribe();

        expect(component.weiterClickEmitter.emit).toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    describe('create document button', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, createDocumentButtonTestId);
      });

      it('should have been called with inputs', () => {
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();
        const createDocumentButton: BescheidWizardCreateDocumentButtonContainerComponent =
          getElementComponentFromFixtureByCss<BescheidWizardCreateDocumentButtonContainerComponent>(
            fixture,
            createDocumentButtonTestId,
          );

        expect(createDocumentButton.bescheidResource).toEqual(component.bescheidResource);
      });
    });

    describe('upload document button', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, uploadDocumentButtonTestId);
      });

      it('should have been called with inputs', () => {
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();
        const uploadDocumentButton: BescheidWizardUploadDocumentButtonContainerComponent =
          getElementComponentFromFixtureByCss<BescheidWizardUploadDocumentButtonContainerComponent>(
            fixture,
            uploadDocumentButtonTestId,
          );

        expect(uploadDocumentButton.bescheidResource).toEqual(component.bescheidResource);
      });
    });

    describe('upload attachment button', () => {
      it('should exists', () => {
        existsAsHtmlElement(fixture, uploadAttachmentButtonTestId);
      });

      it('should have been called with inputs', () => {
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();
        const uploadAttachmentButton: BescheidWizardUploadAttachmentButtonContainerComponent =
          getElementComponentFromFixtureByCss<BescheidWizardUploadAttachmentButtonContainerComponent>(
            fixture,
            uploadDocumentButtonTestId,
          );

        expect(uploadAttachmentButton.bescheidResource).toEqual(component.bescheidResource);
      });
    });

    describe('weiter button', () => {
      it('should exists', () => {
        component.bescheidResource = createBescheidResource([BescheidLinkRel.UPDATE]);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, weiterButtonTestId);
      });

      it('should NOT exists', () => {
        component.bescheidResource = createBescheidResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, weiterButtonTestId);
      });

      it('should have been called with inputs', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        component.submitStateResource$ = of(commandStateResource);
        component.bescheidResource = createBescheidResource([BescheidLinkRel.UPDATE]);

        fixture.detectChanges();
        const weiterButton: BescheidWizardWeiterButtonComponent =
          getElementComponentFromFixtureByCss<BescheidWizardWeiterButtonComponent>(fixture, weiterButtonTestId);
        component.submitStateResource$.subscribe();

        expect(weiterButton.submitStateResource).toEqual(commandStateResource);
      });

      describe('output', () => {
        describe('clickEmitter', () => {
          it('should call handler', () => {
            component.gotoNextStep = jest.fn();
            component.bescheidResource = createBescheidResource([BescheidLinkRel.UPDATE]);
            fixture.detectChanges();

            triggerEvent({ fixture, elementSelector: weiterButtonTestId, name: 'clickEmitter' });

            expect(component.gotoNextStep).toHaveBeenCalled();
          });
        });
      });
    });
  });
});
